<?php
/**
 * class-wgp-variations-admin.php
 *
 * Copyright (c) "eggemplo" Antonio Blanco www.eggemplo.com
 *
 * This code is provided subject to the license granted.
 * Unauthorized use and distribution is prohibited.
 * See COPYRIGHT.txt and LICENSE.txt
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * This header and all notices must be kept intact.
 * 
 * @author Antonio Blanco
 * @package woo-group-pricing
 * @since 2.3
 */
if (! defined ( 'ABSPATH' )) {
	exit ();
}

/**
 * Category product admin handlers.
 */
class WGP_Variations_Admin {

	/**
	 * Sets up the init action.
	 */
	public static function init() {

		// Display Fields
		add_action ( 'woocommerce_product_after_variable_attributes', array (
				__CLASS__,
				'woocommerce_product_after_variable_attributes' 
		), 10, 3 );
		// JS to add fields for new variations
		add_action ( 'woocommerce_product_after_variable_attributes_js', array (
				__CLASS__,
				'woocommerce_product_after_variable_attributes' 
		) );
		// Save variation fields
		add_action ( 'woocommerce_process_product_meta_variable', array (
				__CLASS__,
				'woocommerce_process_product_meta_variable' 
		), 10, 1 );

		// filter <del> tags for variable products
// 		add_filter('woocommerce_variable_sale_price_html', array ( __CLASS__, 'woocommerce_variable_sale_price_html' ), 10, 2 );
// 		add_filter('woocommerce_variable_sale_price', array ( __CLASS__, 'woocommerce_variable_sale_price_html' ), 10, 2 );

		// Variations
		add_filter( 'woocommerce_get_variation_price', array ( __CLASS__, 'woocommerce_get_variation_price' ), 10, 4);

		// Get variation price. Added Woo 2.4
		add_filter( 'woocommerce_variation_prices_price', array ( __CLASS__, 'woocommerce_variation_prices' ), 10, 3);
		add_filter( 'woocommerce_variation_prices_regular_price', array ( __CLASS__, 'woocommerce_variation_prices' ), 10, 3);
		add_filter( 'woocommerce_variation_prices_sale_price', array ( __CLASS__, 'woocommerce_variation_prices' ), 10, 3);

		// Remove cache added in Woocommerce 2.4
		add_filter('woocommerce_get_variation_prices_hash', array ( __CLASS__, 'woocommerce_get_variation_prices_hash' ), 10, 3);

		add_action( 'woocommerce_save_product_variation', array( __CLASS__, 'woocommerce_save_product_variation' ), 10, 2 );

	}

	/**
	 * Remove cache added in Woocommerce 2.4 relating to variation prices.
	 * @param array $cache_key_args
	 * @param unknown $product
	 * @param unknown $display
	 * @return array
	 */
	public static function woocommerce_get_variation_prices_hash ( $hash, $product, $display ) {
		// Delete the cached data, if it exists
		$cache_key = 'wc_var_prices' . substr( md5( json_encode( $hash ) ), 0, 22 ) . WC_Cache_Helper::get_transient_version( 'product' );

		delete_transient($cache_key);
		// Woocommerce 2.5
		delete_transient('wc_var_prices_' . $product->get_id());
		return $hash;
	}

	public static function woocommerce_product_after_variable_attributes ($loop, $variation_data, $variation) {
		?>
		<tr>
			<td style="background:none repeat scroll 0 0 #fafafa;">
				<div>
					<h3>Groups Pricing</h3>
					<?php self::woocommerce_product_variations_write_panels($loop, $variation_data, $variation); ?>
				</div>
			</td>
		</tr>
		<?php
	}

	public static function woocommerce_process_product_meta_variable($post_id) {
		if (isset ( $_POST ['variable_sku'] )) {
			$variable_sku = $_POST ['variable_sku'];
			$variable_post_id = $_POST ['variable_post_id'];
			$groups = WooGroupPricing::get_all_groups();

			for( $i = 0; $i < sizeof ( $variable_sku ); $i ++ ) {
				$variation_id = ( int ) $variable_post_id [$i];

				if ( $groups ) {
					foreach ( $groups as $group ) {
						if ( isset ( $_POST['groups_pricing_value_' . $group->group_id . "_" . $i] ) &&
								( $_POST['groups_pricing_value_' . $group->group_id . "_" . $i] !== "" ) ) {
							$variable_custom_field = $_POST['groups_pricing_value_' . $group->group_id . "_" . $i];
							update_post_meta( $variation_id, 'groups_pricing_value_' . $group->group_id, ( $variable_custom_field !== "" ) ? trim(str_replace(",", ".", $variable_custom_field)) : '' );
						} else {
							delete_post_meta( $variation_id, 'groups_pricing_value_' . $group->group_id );
						}
					}
				}
			}
		}
	}

	public static function woocommerce_save_product_variation ( $variation_id, $i ) {
		if (isset ( $_POST ['variable_sku'] )) {
			$variable_sku = $_POST ['variable_sku'];
			$variable_post_id = $_POST ['variable_post_id'];
			$groups = WooGroupPricing::get_all_groups();

			for( $i = 0; $i < sizeof ( $variable_sku ); $i ++ ) {
				$variation_id = ( int ) $variable_post_id [$i];
				if ( $groups ) {
					foreach ( $groups as $group ) {
						if ( isset ( $_POST['groups_pricing_value_' . $group->group_id . "_" . $i] ) &&
								( $_POST['groups_pricing_value_' . $group->group_id . "_" . $i] !== "" ) ) {
									$variable_custom_field = $_POST['groups_pricing_value_' . $group->group_id . "_" . $i];
									update_post_meta( $variation_id, 'groups_pricing_value_' . $group->group_id, ( $variable_custom_field !== "" ) ? trim(str_replace(",", ".", $variable_custom_field)) : '' );
								} else {
									delete_post_meta( $variation_id, 'groups_pricing_value_' . $group->group_id );
								}
					}
				}
			}
		}
		// clear product cache
		WooGroupPricing::clear_products_cache();
	}

	public static function woocommerce_product_variations_write_panels($loop, $variation_data, $variation) {
		global $post;

		$groups = WooGroupPricing::get_all_groups();

		foreach ( $groups as $group ) {
			$value = get_post_meta( $variation->ID, 'groups_pricing_value_' . $group->group_id, true );
			$value = ( $value !== null )?get_post_meta( $variation->ID, 'groups_pricing_value_' . $group->group_id, true ):"";
		?>
		<div class="wgp-variation-field">
			<label><?php echo $group->name; ?>:</label> 
			<input
				type="text" size="5" name="groups_pricing_value_<?php echo $group->group_id;?>_<?php echo $loop; ?>"
				value="<?php echo $value;?>" />
		</div>
		<?php 
		}
	}

	/**
	 * Filter <del> tabs for variable products
	 * @param String $pricehtml
	 * @param Object $product
	 * @return String
	 */
// 	public static function woocommerce_variable_sale_price_html ( $pricehtml, $product ) {
// 		global $post, $woocommerce;

// 		$string = $pricehtml;

// 		if ( ($post == null) || !is_admin() ) {
// 			$commission = WooGroupPricing::get_commission( $product );
// 		//	if ( $commission ) { // if applying  discount, then remove the original prices.
// 			$method = get_option( "wgp-method", "rate" );
// 			if ( ( ( $method == "rate" ) && ( $commission < 1 ) ) || ( ( $method !== "rate" ) && ( $commission > 0 ) ) ) {
// 				$string=preg_replace('/<del[^>]*>.+?<\/del>/i', '', $string);
// 			}
// 		}
// 		return $string;
// 	}

	/**
	 * Get woocommerce variation price discounted. Called the show the variation range price (category lists and product page).
	 * @param unknown $price
	 * @param unknown $variation
	 * @param unknown $product
	 * @return float
	 */
	public static function woocommerce_variation_prices ( $price, $variation, $product ) {
		global $post, $woocommerce;

		$baseprice = $price;
		$result = $baseprice;

		if ( is_user_logged_in() && ( ( $post == null ) || !is_admin() ) ) {
			if ( version_compare( $woocommerce->version, '3.0.0', '>=' ) ) {
				$variation_id = $variation->get_id();
			} else {
				$variation_id = $variation->variation_id;
			}
			if ( ! $variation_id ) {
				return false;
			}
			$commission = self::get_commission( $product, $variation_id );

			if ( $commission !== null ) {
				$method = get_option( "wgp-method", "rate" );
				if ( ( ( $method == "rate" ) && ( $commission <= 1 ) ) || ( ( $method !== "rate" ) && ( $commission >= 0 ) ) ) {
	
					$price = get_post_meta( $variation_id, '_price', true );
					$result = $price;
	
					$baseprice = $variation->get_regular_price();
					if ( version_compare( $woocommerce->version, '3.0.0', '>=' ) ) {
						$variation_get_price = $variation->get_price( 'edit' );
					} else {
						$variation_get_price = $variation->price;
					}
					$baseprice = apply_filters( 'woo_group_pricing_variation_baseprice', $variation->get_regular_price(), $product, $variation );
					if ( $variation->get_sale_price() != $variation->get_regular_price() && $variation->get_sale_price() == $variation_get_price ) {
						if ( get_option( "wgp-baseprice", "regular" )=="sale" ) {
							$baseprice = $variation->get_sale_price();
						}
					}

					$product_price = apply_filters( 'woo_group_pricing_variation_product_price', $baseprice, $product, $variation );

					$type = get_option( "wgp-method", "rate" );
					$result = 0;
					if ($type == "rate") {
						// if rate and price includes taxes
						if ( $variation->is_taxable() && get_option('woocommerce_prices_include_tax') == 'yes' ) {
							$_tax       = new WC_Tax();
							$tax_rates  = $_tax->get_shop_base_rate( $variation->tax_class );
							$taxes      = $_tax->calc_tax( $baseprice, $tax_rates, true );
							$product_price      = $_tax->round( $baseprice - array_sum( $taxes ) );
						}
	
						$result = WooGroupPricing::bcmul($product_price, $commission, get_option( 'wgp-decimals', 4 ) );
	
						if ( $variation->is_taxable() && get_option('woocommerce_prices_include_tax') == 'yes' ) {
							$_tax       = new WC_Tax();
							$tax_rates  = $_tax->get_shop_base_rate( $variation->tax_class );
							$taxes      = $_tax->calc_tax( $result, $tax_rates, false ); // important false
							$result      = $_tax->round( $result + array_sum( $taxes ) );
						}
					} else {
						if ( get_option( "wgp-haveset", "discounts" ) === 'discounts' ) {
							$result = WooGroupPricing::bcsub($product_price, $commission, get_option( 'wgp-decimals', 4 ) );
						} else {
							$result = $commission;
						}
					}
				}
			}
		}

		return $result;
	}

	/**
	 * Get variation price discounted. Used when you change between options.
	 * @param unknown $price
	 * @param unknown $product
	 * @param unknown $min_or_max
	 * @param unknown $display
	 * @return float
	 */
	public static function woocommerce_get_variation_price ( $price, $product, $min_or_max, $display ) {
		global $post, $woocommerce;

		$baseprice = $price;
		$result = $baseprice;

		if ( is_user_logged_in() && ( ( $post == null ) || !is_admin() ) ) {
			$variation_id = get_post_meta( $product->get_id(), '_' . $min_or_max . '_price_variation_id', true );

			if ( ! $variation_id ) {
				return false;
			}

			$price = get_post_meta( $variation_id, '_price', true );
			$result = $price;

			if ( $display ) {
				$variation = $product->get_child( $variation_id );

				$commission = self::get_commission( $product, $variation_id );

				$method = get_option( "wgp-method", "rate" );
				if ( ( ( $method == "rate" ) && ( $commission < 1 ) ) || ( ( $method !== "rate" ) && ( $commission > 0 ) ) ) {

					$baseprice = apply_filters( 'woo_group_pricing_get_variation_baseprice', $variation->get_regular_price(), $product, $variation );
					if ( version_compare( $woocommerce->version, '3.0.0', '>=' ) ) {
						$variation_get_price = $variation->get_price( 'edit' );
					} else {
						$variation_get_price = $variation->price;
					}
					if ( $variation->get_sale_price() != $variation->get_regular_price() && $variation->get_sale_price() == $variation_get_price ) {
						if ( get_option( "wgp-baseprice", "regular" )=="sale" ) {
							$baseprice = $variation->get_sale_price();
						}
					}
					$product_price = apply_filters( 'woo_group_pricing_get_variation_product_price', $baseprice, $product, $variation );

					$type = get_option( "wgp-method", "rate" );
					$result = 0;
					if ($type == "rate") {
						// if rate and price includes taxes
						if ( $variation->is_taxable() && get_option('woocommerce_prices_include_tax') == 'yes' ) {
							$_tax       = new WC_Tax();
							$tax_rates  = $_tax->get_shop_base_rate( $variation->tax_class );
							$taxes      = $_tax->calc_tax( $baseprice, $tax_rates, true );
							$product_price      = $_tax->round( $baseprice - array_sum( $taxes ) );
						}

						$result = WooGroupPricing::bcmul($product_price, $commission, get_option( 'wgp-decimals', 4 ) );

						if ( $variation->is_taxable() && get_option('woocommerce_prices_include_tax') == 'yes' ) {
							$_tax       = new WC_Tax();
							$tax_rates  = $_tax->get_shop_base_rate( $variation->tax_class );
							$taxes      = $_tax->calc_tax( $result, $tax_rates, false ); // important false
							$result      = $_tax->round( $result + array_sum( $taxes ) );
						}
					} else {
						if ( get_option( "wgp-haveset", "discounts" ) === 'discounts' ) {
							$result = WooGroupPricing::bcsub($product_price, $commission, get_option( 'wgp-decimals', 4 ) );
						} else {
							$result = $commission;
						}
					}
				}
			}
		}
		return $result;
	}

	/**
	 * Calculates the commissions.
	 * Order by priority:
	 * 1.- Variation values
	 * 2.- Product values
	 * 3.- Category values
	 * 4.- Default value
	 * @param unknown $product
	 * @param unknown $variation_id
	 * @return number
	 */
	public static function get_commission ( $product, $variation_id, $group = null ) {
		global $post, $woocommerce;

		$user_id = get_current_user_id();
		$user_groups = WooGroupPricing::get_user_groups ( $user_id );
		$discount = 0;
		if ( sizeof( $user_groups ) > 0 ) {
			if ( isset( $group ) ) {
				$first_group = $group;
			} else {
				$first_group = self::get_user_group( $user_groups, $product, $variation_id );
			}

			// Variation - custom discount ...
			$custom = get_post_meta( $variation_id, 'groups_pricing_value_' . $first_group->group_id, true);

			if ( $custom !== "" ) {
				$discount = $custom;
			} else {

				// Product - custom discount ...
				$custom = get_post_meta( $product->get_id(), 'groups_pricing_value_' . $first_group->group_id, true);

				// Category - custom discount ....
				if ( $custom !== "" ) {
					$discount = $custom;
				} else {
					$categories = wp_get_post_terms( $product->get_id(), 'product_cat',array('fields'=>'ids') );
					if ( sizeof( $categories ) > 0 ) {
						$max_cat_id = null;
						$max_cat_discount = 0;
						foreach ( $categories as $cat_id ) {
							$cat_discount = get_woocommerce_term_meta($cat_id, 'groups_pricing_value_' . $first_group->group_id, true);
							if ( $cat_discount !== "" ) {
								if ( $cat_discount > $max_cat_discount ) {
									$max_cat_discount = $cat_discount;
									$max_cat_id = $cat_id;
								}
							}
						}
						if ( $max_cat_id !== null ) {
							$custom = $max_cat_discount;
						}
					}
				}
			}

			// general discount ....
			if ( $custom !== "" ) {
				$discount = $custom;
			} else {
				if ( get_option( "wgp-" . $first_group->group_id, "-1" ) !== "-1" ) {
					$discount = get_option( "wgp-" . $first_group->group_id, "" );
				}
			}
		}

		if ( $discount == "" ) {
			$discount = null;
		} else {
			$method = get_option( "wgp-method", "rate" );
			if ( $method == "rate" ) {
				if ( get_option( "wgp-haveset", "discounts" ) === 'discounts' ) {
					$discount = WooGroupPricing::bcsub ( 1, $discount, get_option( 'wgp-decimals', 4 ) );
				}
			}
		}

		return $discount;
	}

	/**
	 * Get the group applied to an user.
	 * @param array $groups
	 * @return group_id or null if an error occurs.
	 */
	public static function get_user_group( $groups = array(), $product, $variation_id ) {
		$result = null;

		if ( sizeof( $groups ) > 0 ) {
			$option = get_option( "wgp-ifseveral", "first" );

			if ( sizeof( $groups ) == 1 ) {
				$result = $groups[0];
			} else {
				switch ( $option ) {
					case 'higher':
						$result = null;
						$commission = null;
						$method = get_option( "wgp-method", "rate" );
						foreach ( $groups as $group ) {
							$temp_com = self::get_commission( $product, $variation_id, $group );
							if ( ( $temp_com !== null ) && ( $temp_com !== '' ) ) {
								if ( $method == 'rate' ) {
									if ( ( $commission == null ) || ( $temp_com < $commission ) ) {
										$commission = $temp_com;
										$result = $group;
									}
								} else {
									if ( ( $commission == null ) || ( $temp_com > $commission ) ) {
										$commission = $temp_com;
										$result = $group;
									}
								}
							}
						}
						if ( $result == null ) {
							$result = $groups[0];
						}
						break;
					case 'lower':
						$result = null;
						$commission = null;
						$method = get_option( "wgp-method", "rate" );
						foreach ( $groups as $group ) {
							$temp_com = self::get_commission( $product, $variation_id, $group );
							if ( ( $temp_com !== null ) && ( $temp_com !== '' ) ) {
								if ( $method == 'rate' ) {
									if ( ( $commission == null ) || ( $temp_com > $commission ) ) {
										$commission = $temp_com;
										$result = $group;
									}
								} else {
									if ( ( $commission == null ) || ( $temp_com < $commission ) ) {
										$commission = $temp_com;
										$result = $group;
									}
								}
							}
						}
						if ( $result == null ) {
							$result = $groups[0];
						}
						break;
					case 'last':
						$result = $groups[sizeof($groups)-1];
						break;
					default:
					case 'first':
						$result = $groups[0];
						break;
				}
			}
		}
		return $result;
	}

}
WGP_Variations_Admin::init();