<?php
/**
 * class-woogrouppricing.php
 *
 * Copyright (c) Antonio Blanco http://www.blancoleon.com
 *
 * This code is released under the GNU General Public License.
 * See COPYRIGHT.txt and LICENSE.txt.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * This header and all notices must be kept intact.
 *
 * @author Antonio Blanco
 * @package woogrouppricing
 * @since woogrouppricing 1.0.0
 */

/**
 * WooGroupPricing class
 */
class WooGroupPricing {

	const DEFAULT_BOTH_PRICES_TEXT = "Original price: [original_price], Your price: [discounted_price]";

	public static function init() {
		global $woocommerce;

		if ( version_compare( $woocommerce->version, '3.0.0', '>=' ) ) {
			add_filter('woocommerce_product_get_price', array( __CLASS__, 'woocommerce_get_price' ), 10, 2);
		} else {
			add_filter('woocommerce_get_price', array( __CLASS__, 'woocommerce_get_price' ), 10, 2);
		}

		// Cart 3.x
		add_filter('woocommerce_cart_product_price', array( __CLASS__, 'woocommerce_get_price' ), 10, 2);
		add_filter('woocommerce_product_variation_get_price', array( __CLASS__, 'woocommerce_get_price' ), 10, 2);

		add_action('woocommerce_product_write_panel_tabs', array(__CLASS__,'woocommerce_product_write_panel_tabs') );
		add_action('woocommerce_product_write_panels', array(__CLASS__,'woocommerce_product_write_panels') );
		add_action('woocommerce_process_product_meta', array(__CLASS__,'woocommerce_process_product_meta') );

		add_filter( 'woocommerce_get_price_html', array(__CLASS__,'woocommerce_get_price_html'), 100, 2 );
	}

	public static function woocommerce_get_price ( $price, $product ) {
		global $post, $woocommerce;

		$baseprice = $price;
		$result = $baseprice;

		if ( is_user_logged_in() && ( ( $post == null ) || !is_admin() ) ) {

			$commission = null;
			if ( $product->is_type( 'variation' ) ) {
				if ( version_compare( $woocommerce->version, '3.0.0', '>=' ) ) {
					$commission = WGP_Variations_Admin::get_commission( $product, $product->get_id() );
				} else {
					$commission = WGP_Variations_Admin::get_commission( $product, $product->variation_id );
				}
			} else {
				$commission = self::get_commission( $product );
			}

			$method = get_option( 'wgp-method', 'rate' );
			$values_are = get_option( 'wgp-haveset', 'discounts' );

			if ( $commission !== null ) {
				if ( ( ( $method == "rate" ) && ( $commission <= 1 ) ) || ( ( $method !== "rate" ) && ( $commission >= 0 ) ) ) {

					$baseprice = $product->get_regular_price();

					// get_price( $context ) We need to set 'edit', so the woocommerce_product_get_price filter is not hooked
					if ( version_compare( $woocommerce->version, '3.0.0', '>=' ) ) {
						$product_get_price = $product->get_price( 'edit' );
					} else {
						$product_get_price = $product->price;
					}

					if ( $product->get_sale_price() != $product->get_regular_price() && $product->get_sale_price() == $product_get_price ) {
						if ( get_option( "wgp-baseprice", "regular" )=="sale" ) {
							$baseprice = $product->get_sale_price();
						}
					}
					$product_price = apply_filters( 'woo_group_pricing_product_price', $baseprice, $product );

					$type = get_option( "wgp-method", "rate" );
					$result = 0;
					if ($type == "rate") {
						// if rate and price includes taxes
						if ( $product->is_taxable() && get_option('woocommerce_prices_include_tax') == 'yes' ) {
							$_tax       = new WC_Tax();
							$tax_rates  = $_tax->get_shop_base_rate( $product->tax_class );
							$taxes      = $_tax->calc_tax( $baseprice, $tax_rates, true );
							$product_price      = $_tax->round( $baseprice - array_sum( $taxes ) );
						}

						$result = self::bcmul($product_price, $commission, get_option( 'wgp-decimals', 4 ) );

						if ( $product->is_taxable() && get_option('woocommerce_prices_include_tax') == 'yes' ) {
							$_tax       = new WC_Tax();
							$tax_rates  = $_tax->get_shop_base_rate( $product->tax_class );
							$taxes      = $_tax->calc_tax( $result, $tax_rates, false ); // important false
							$result      = $_tax->round( $result + array_sum( $taxes ) );
						}
					} else {
						if ( get_option( "wgp-haveset", "discounts" ) === 'discounts' ) {
							$result = self::bcsub($product_price, $commission, get_option( 'wgp-decimals', 4 ) );
						} else {
							$result = $commission;
						}
					}
				}
			}
		}

		return $result;
	}

	public static function woocommerce_product_write_panel_tabs() {

		echo '<li class="groups_pricing_tab general_options"><a href="#groups_pricing_data">' . __( 'Groups Pricing', WOO_GROUP_PRICING_DOMAIN ) . '</a></li>';

	}

	public static function woocommerce_product_write_panels() {
		global $post;

		if( function_exists('wc_get_product') ){
			$product = wc_get_product( $post->ID );
		} else {
			$product = get_product( $post->ID );
		}
		if( $product->is_type( 'variable' ) ){
			// do something with external products
			echo '<div id="groups_pricing_data" class="panel woocommerce_options_panel">';
			echo "<p>In Variable Products these fields are in 'Variations' tabs.</p>";
			echo '</div>';
			return;
		}

		$groups = self::get_all_groups();
		if ( $groups ) {
			$pricing_options = array();
			foreach ( $groups as $group ) {
				$pricing_options['value_' . $group->group_id] = get_post_meta($post->ID, 'groups_pricing_value_' . $group->group_id, true);
			}
		}
		?>
		<div id="groups_pricing_data" class="panel woocommerce_options_panel">
			<div class="options_group">
				<p class="description">
					<?php echo __( 'Leave empty if no custom group discount should be applied to this product (default setting).', WOO_GROUP_PRICING_DOMAIN ); ?>
				</p>
			</div>

			<div class="options_group custom_tab_options">
				<?php 
				foreach ( $groups as $group ) {
				?>
				<p class="form-field">
					<label><?php echo $group->name; ?>:</label>
					<input type="text" class="short" name="groups_pricing_value_<?php echo $group->group_id;?>" value="<?php echo @$pricing_options['value_' . $group->group_id]; ?>" />
				</p>
				<?php
				}
				?>
			</div>
		</div>
	<?php
	}

	public static function woocommerce_process_product_meta( $post_id ) {
		$groups = self::get_all_groups();
		if ( $groups ) {
			foreach ( $groups as $group ) {
				if ( isset($_POST['groups_pricing_value_' . $group->group_id]) && ( $_POST['groups_pricing_value_' . $group->group_id] !== "" ) ) {
					update_post_meta( $post_id, 'groups_pricing_value_' . $group->group_id, ( isset($_POST['groups_pricing_value_' . $group->group_id]) && ( $_POST['groups_pricing_value_' . $group->group_id] !== "" ) ) ? trim(str_replace(",", ".", $_POST['groups_pricing_value_' . $group->group_id])) : '' );
				} else {
					delete_post_meta( $post_id, 'groups_pricing_value_' . $group->group_id );
				}
			}
		}
		// clear product cache
		WooGroupPricing::clear_products_cache();
	}

	public static function get_commission ( $product, $group = null ) {
		global $post, $woocommerce;

		$discount = null;
		if ( is_user_logged_in() ) {
			$user_id = get_current_user_id();
			$user_groups = self::get_user_groups ( $user_id );
			if ( sizeof( $user_groups ) > 0 ) {
				if ( isset( $group ) ) {
					$first_group = $group;
				} else {
					$first_group = self::get_user_group( $user_groups, $product );
				}
				$custom = "";
				if ( $product !== null ) {
					// Product - custom discount ...
					$custom = get_post_meta( $product->get_id(), 'groups_pricing_value_' . $first_group->group_id, true);

					// Category - custom discount ....
					if ( $custom !== "" ) {
						$discount = $custom;
					} else {
						$categories = wp_get_post_terms( $product->get_id(), 'product_cat',array('fields'=>'ids') );
						if ( sizeof( $categories ) > 0 ) {
							$max_cat_id = null;
							$max_cat_discount = 0;
							foreach ( $categories as $cat_id ) {
								$cat_discount = get_woocommerce_term_meta($cat_id, 'groups_pricing_value_' . $first_group->group_id, true);
								if ( $cat_discount !== "" ) {
									if ( $cat_discount > $max_cat_discount ) {
										$max_cat_discount = $cat_discount;
										$max_cat_id = $cat_id;
									}
								}
							}
							if ( $max_cat_id !== null ) {
								$custom = $max_cat_discount;
							}
						}
					}
				}
				// general discount ....
				if ( $custom !== "" ) {
					$discount = $custom;
				} else {
					if ( get_option( "wgp-" . $first_group->group_id, "-1" ) !== "-1" ) {
						$discount = get_option( "wgp-" . $first_group->group_id, "" );
					}
				}
			}
		}

		if ( $discount == "" ) {
			$discount = null;
		} else {
			$method = get_option( "wgp-method", "rate" );
			if ( $method == "rate" ) {
				if ( get_option( "wgp-haveset", "discounts" ) === 'discounts' ) {
					$discount = self::bcsub ( 1, $discount, get_option( 'wgp-decimals', 4 ) );
				}
			}
		}

		return $discount;
	}

	public static function woocommerce_get_price_html( $price, $product ){
		global $post;
		$result = $price;

		if ( is_user_logged_in() && !is_admin() ) {

			if ( get_option( "wgp-baseprice", "regular" ) == "sale" ) {
				$price_key = '_price';
			} else {
				$price_key = '_regular_price';
			}

			if ( ( $product->get_type() == 'variable' ) || ( $product->get_type() == 'variation' ) ) {

				$commission = null;
				$original_prices = array();

				if ( $product->get_type() == 'variable' ) {
					$children = $product->get_children(true);

					foreach ( $children as $child ) {
						$original_prices[] = get_post_meta( $child, $price_key, true );
						if ( $commission == null ) {
							$commission = WGP_Variations_Admin::get_commission( $product, $child );
						}
					}
				} else {  // 'variation' on javascript variable product select
					//$original_prices[] = get_post_meta( $product, $price_key, true );
					if ( get_option( "wgp-baseprice", "regular" ) == "sale" ) {
						$original_prices[] = $product->get_sale_price();
						$price = wc_price( self::woocommerce_get_price($product->get_sale_price(), $product) );
					} else {
						$original_prices[] = $product->get_regular_price();
						$price = wc_price( self::woocommerce_get_price($product->get_regular_price(), $product) );
					}
					if ( $commission == null ) {
						$commission = WGP_Variations_Admin::get_commission( $product, $product->get_id() );
					}
					// In woocommerce_get_price_html called from variation (select of variable) the price is not discounted
					$result = $price;
				}

				$original_price = $price;

				if ( $commission !== null ) {
					// if the product price includes the taxes, then we need to subtract it.
					foreach ( $original_prices as $key=>$original_p ) {
						if ( wc_tax_enabled() && ( 'excl' === get_option( 'woocommerce_tax_display_shop' ) ) && ( get_option('woocommerce_prices_include_tax') == 'yes' ) ) {
							$_tax           = new WC_Tax();
							$tax_rates      = $_tax->get_base_tax_rates( $product->tax_class );
							$taxes          = $_tax->calc_tax( $original_p, $tax_rates, true );
							$original_prices[$key] = $_tax->round( $original_p - array_sum( $taxes ) );
						}
					}

					$min_price = min( $original_prices );
					$max_price = max( $original_prices );

					$original_price = $min_price !== $max_price ? sprintf( _x( '%1$s–%2$s', 'Price range: from-to', 'woocommerce' ), wc_price( $min_price ), wc_price( $max_price ) ) : wc_price( $min_price );
				}
			} else {

				$original_price = get_post_meta( $product->get_id(), $price_key, true );
				$commission = WooGroupPricing::get_commission( $product );

				if ( $commission !== null ) {
					// if the product price includes the taxes, then we need to subtract it.
					if ( wc_tax_enabled() && ( 'excl' === get_option( 'woocommerce_tax_display_shop' ) ) && ( get_option('woocommerce_prices_include_tax') == 'yes' ) ) {
						$_tax           = new WC_Tax();
						$tax_rates      = $_tax->get_base_tax_rates( $product->tax_class );
						$taxes          = $_tax->calc_tax( get_post_meta( $product->get_id(), $price_key, true ), $tax_rates, true );
						$original_price = $_tax->round( get_post_meta( $product->get_id(), $price_key, true ) - array_sum( $taxes ) );
					}
					$original_price = wc_price( $original_price );
				}
			}

			$method = get_option( "wgp-method", "rate" );
			if ( ( ( $method == "rate" ) && ( $commission < 1 ) ) || ( ( $method !== "rate" ) && ( $commission > 0 ) ) ) {

				$text = get_option( "wgp-bothpricestext", WooGroupPricing::DEFAULT_BOTH_PRICES_TEXT );

				// If $price has <del> tag ( on sale ), we remove this.
				$price=preg_replace('/<del[^>]*>.+?<\/del>/i', '', $price);

				$tokens = array();
				$tokens['original_price'] = $original_price;
				$tokens['discounted_price'] = $price;

				// You can add extra tokens
				$tokens = apply_filters( 'woo_group_pricing_both_prices_tokens', $tokens, $product, $commission, $original_price, $price );

				$text = self::substitute_tokens( $text, $tokens );

				if ( get_option( "wgp-displayoriginal", 0 ) == 1 ) {
					// correct filter name
					$result = apply_filters(
						'woo_group_pricing_display_original_prices',
						$text,
						$original_price,
						$price
					);
				}
			}
		}

		return $result;
	}

	private static function substitute_tokens( $s, $tokens ) {
		foreach ( $tokens as $key => $value ) {
			if ( key_exists( $key, $tokens ) ) {
				$substitute = $tokens[$key];
				$s = str_replace( "[" . $key . "]", $substitute, $s );
			}
		}
		return $s;
	}

	/* Function relating to Groups */

	public static function get_all_groups (  ) {
		global $wpdb;

		$groups_table = _groups_get_tablename( 'group' );

		return $wpdb->get_results( "SELECT * FROM $groups_table ORDER BY name" );

	}

	/**
	 * Get all user groups.
	 * @param int $user_id
	 * @return array
	 */
	public static function get_user_groups( $user_id ) {
		global $wpdb;

		$groups_table = _groups_get_tablename( 'group' );
		$result = array();
		if ( $groups = $wpdb->get_results( "SELECT * FROM $groups_table ORDER BY group_id DESC" ) ) {
			foreach( $groups as $group ) {
				$is_member = Groups_User_Group::read( $user_id, $group->group_id ) ? true : false;
				if ( $is_member ) {
					$result[] = $group;
				}
			}
		}
		return  $result;
	}

	/**
	 * Get the group applied to an user.
	 * @param array $groups
	 * @return group_id or null if an error occurs.
	 */
	public static function get_user_group( $groups = array(), $product, $user_id = null ) {
		$result = null;

		if ( $user_id == null ) {
			$user_id = get_current_user_id();
		}
		if ( sizeof( $groups ) > 0 ) {
			$option = get_option( "wgp-ifseveral", "first" );

			if ( sizeof( $groups ) == 1 ) {
				$result = $groups[0];
			} else {
				switch ( $option ) {
					case 'higher':
						$result = null;
						$commission = null;
						$method = get_option( "wgp-method", "rate" );
						foreach ( $groups as $group ) {
							$temp_com = self::get_commission( $product, $group );
							if ( ( $temp_com !== null ) && ( $temp_com !== '' ) ) {
								if ( $method == 'rate' ) {
									if ( ( $commission == null ) || ( $temp_com < $commission ) ) {
										$commission = $temp_com;
										$result = $group;
									}
								} else {
									if ( ( $commission == null ) || ( $temp_com > $commission ) ) {
										$commission = $temp_com;
										$result = $group;
									}
								}
							}
						}
						if ( $result == null ) {
							$result = $groups[0];
						}
						break;
					case 'lower':
						$result = null;
						$commission = null;
						$method = get_option( "wgp-method", "rate" );
						foreach ( $groups as $group ) {
							$temp_com = self::get_commission( $product, $group );
							if ( ( $temp_com !== null ) && ( $temp_com !== '' ) ) {
								if ( $method == 'rate' ) {
									if ( ( $commission == null ) || ( $temp_com > $commission ) ) {
										$commission = $temp_com;
										$result = $group;
									}
								} else {
									if ( ( $commission == null ) || ( $temp_com < $commission ) ) {
										$commission = $temp_com;
										$result = $group;
									}
								}
							}
						}
						if ( $result == null ) {
							$result = $groups[0];
						}
						break;
					case 'last':
						$result = $groups[sizeof($groups)-1];
						break;
					default:
					case 'first':
						$result = $groups[0];
						break;
				}
			}
		}
		return $result;
	}

	/* Math functions */

	public static function bcmul( $data1, $data2, $prec = 0 ) {
		$result = 0;
		if ( function_exists('bcmul') ) {
			$result = bcmul( $data1, $data2, $prec );
		} else {
			$value = $data1 * $data2;
			if ($prec) {
				$result = round($value, $prec);
			}
		}
		return $result;
	}

	public static function bcsub( $data1, $data2, $prec = 0 ) {
		$result = 0;
		if ( function_exists('bcsub') ) {
			$result = bcsub( $data1, $data2, $prec );
		} else {
			$value = $data1 - $data2;
			if ($prec) {
				$result = round($value, $prec);
			}
		}
		return $result;
	}

	/**
	 * Clear product cache
	 * @since WC 2.4
	 */
	public static function clear_products_cache() {
		global $woocommerce;
		if(version_compare($woocommerce->version, '2.4', '>=')) {
			WC_Cache_Helper::get_transient_version('product', true);
		}
	}
}
WooGroupPricing::init();