<?php
/**
 * woo-group-pricing.php
 *
 * Copyright (c) 2011,2012 Antonio Blanco http://www.blancoleon.com
 *
 * This code is released under the GNU General Public License.
 * See COPYRIGHT.txt and LICENSE.txt.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * This header and all notices must be kept intact.
 *
 * @author Antonio Blanco
 * @package woogrouppricing
 * @since woogrouppricing 1.0.0
 *
 * Plugin Name: Woocommerce Group Pricing
 * Plugin URI: http://www.eggemplo.com/plugins/woocommerce-group-pricing
 * Description: Shows different prices according to the user's group
 * Version: 3.3.1
 * Author: eggemplo
 * Author URI: http://www.eggemplo.com
 * License: GPLv3
 */

define( 'WOO_GROUP_PRICING_DOMAIN', 'woogrouppricing' );
define( 'WOO_GROUP_PRICING_PLUGIN_NAME', 'woocommerce-group-pricing' );

define( 'WOO_GROUP_PRICING_FILE', __FILE__ );

if ( !defined( 'WOO_GROUP_PRICING_CORE_DIR' ) ) {
	define( 'WOO_GROUP_PRICING_CORE_DIR', WP_PLUGIN_DIR . '/woocommerce-group-pricing/core' );
}

define( 'WOO_GROUP_PRICING_PLUGIN_URL', plugin_dir_url( WOO_GROUP_PRICING_FILE ) );

class WooGroupPricing_Plugin {

	private static $notices = array();

	public static function init() {

		load_plugin_textdomain( WOO_GROUP_PRICING_DOMAIN, null, WOO_GROUP_PRICING_PLUGIN_NAME . '/languages' );

		register_activation_hook( WOO_GROUP_PRICING_FILE, array( __CLASS__, 'activate' ) );
		register_deactivation_hook( WOO_GROUP_PRICING_FILE, array( __CLASS__, 'deactivate' ) );

		register_uninstall_hook( WOO_GROUP_PRICING_FILE, array( __CLASS__, 'uninstall' ) );

		add_action( 'init', array( __CLASS__, 'wp_init' ) );
		add_action( 'admin_notices', array( __CLASS__, 'admin_notices' ) );

		add_action('admin_head', array( __CLASS__, 'woogrouppricing_enqueue_scripts' ) );
	}

	public static function wp_init() {

		if ( is_multisite() ) {
			$active_sitewide_plugins = array_keys( get_site_option( 'active_sitewide_plugins', array() ) );
			$active_plugins = array_merge( get_option( 'active_plugins', array() ), $active_sitewide_plugins );
		} else {
			$active_plugins = get_option( 'active_plugins', array() );
		}
		$groups_is_active = in_array( 'groups/groups.php', $active_plugins );
		$woo_is_active = in_array( 'woocommerce/woocommerce.php', $active_plugins );

		if ( ( !$groups_is_active ) || ( !$woo_is_active ) ) {
			if ( !$groups_is_active ) {
				self::$notices[] = "<div class='error'>" . __( 'The <strong>Woocommerce Group Pricing</strong> plugin requires the <a href="http://wordpress.org/extend/plugins/groups" target="_blank">Groups</a> plugin to be activated.', WOO_GROUP_PRICING_DOMAIN ) . "</div>";
			}
			if ( !$woo_is_active ) {
				self::$notices[] = "<div class='error'>" . __( 'The <strong>Woocommerce Group Pricing</strong> plugin requires the <a href="http://wordpress.org/extend/plugins/woocommerce" target="_blank">Woocommerce</a> plugin to be activated.', WOO_GROUP_PRICING_DOMAIN ) . "</div>";
			}
			include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
			deactivate_plugins( array( __FILE__ ) );
		} else {

			add_action( 'admin_menu', array( __CLASS__, 'admin_menu' ), 40 );
			//call register settings function
			add_action( 'admin_init', array( __CLASS__, 'register_woogrouppricing_settings' ) );

			if ( !class_exists( "WooGroupPricing" ) ) {
				include_once 'core/class-woogrouppricing.php';
				include_once 'core/class-wgp-categories-admin.php';
				include_once 'core/class-wgp-variations-admin.php';
				include_once 'core/class-wgp-grouped-admin.php';
				include_once 'core/class-wgp-shortcodes.php';
			}
		}
	}

	/**
	 * Register settings as woogrouppricing
	 */
	public static function register_woogrouppricing_settings() {

		register_setting( 'woogrouppricing', 'wgp-method' );
		add_option( 'wgp-method','rate' ); // by default rate

		register_setting( 'woogrouppricing', 'wgp-baseprice' );
		add_option( 'wgp-baseprice','regular' ); // by default regular

		register_setting( 'woogrouppricing', 'wgp-ifseveral' );
		add_option( 'wgp-ifseveral','lower' ); // by default lower discount

		register_setting( 'woogrouppricing', 'wgp-displayoriginal' );
		add_option( 'wgp-displayoriginal', 0 ); // by default 0 = false

		register_setting( 'woogrouppricing', 'wgp-haveset' );
		add_option( 'wgp-haveset','discounts' ); // by default discounts

		register_setting( 'woogrouppricing', 'wgp-decimals' );
		add_option( 'wgp-decimals', apply_filters( 'woo_group_pricing_num_decimals', 4 ) );

	}

	/**
	 * Load scripts.
	 */
	public static function woogrouppricing_enqueue_scripts() {
		wp_register_style( 'wgp-styles', WOO_GROUP_PRICING_PLUGIN_URL . 'css/wgp-admin-styles.css' );
		wp_enqueue_style ('wgp-styles');
	}

	public static function admin_notices() { 
		if ( !empty( self::$notices ) ) {
			foreach ( self::$notices as $notice ) {
				echo $notice;
			}
		}
	}

	/**
	 * Adds the admin section.
	 */
	public static function admin_menu() {
		$admin_page = add_submenu_page(
				'woocommerce',
				__( 'Group Pricing', WOO_GROUP_PRICING_DOMAIN ),
				__( 'Group Pricing', WOO_GROUP_PRICING_DOMAIN ),
				'manage_options',
				'woogrouppricing',
				array( __CLASS__, 'woogrouppricing_settings_menu' )
		);
	}

	/**
	 * Settings menu page.
	 */
	public static function woogrouppricing_settings_menu () {

		$section_links_array = array(
			'method' => __( 'Method', WOO_GROUP_PRICING_DOMAIN ),
			'groups'  => __( 'Groups', WOO_GROUP_PRICING_DOMAIN ),
			'advanced'  => __( 'Advanced', WOO_GROUP_PRICING_DOMAIN )
		);

		$alert = "";

		$section = isset( $_REQUEST['section'] ) ? $_REQUEST['section'] : 'method';

		if ( isset( $_POST['submit'] ) ) {
			switch ( $section ) {
				case 'groups' :
					$groups = WooGroupPricing::get_all_groups();
					foreach ( $groups as $group ) {
						if ( isset( $_POST[ "wgp-" . $group->group_id ] ) && ( $_POST[ "wgp-" . $group->group_id ] !== "" ) ) {
							update_option( "wgp-" . $group->group_id, $_POST[ "wgp-" . $group->group_id ] );
						} else {
							delete_option( "wgp-" . $group->group_id );
						}
					}
					$alert = __("Saved Groups subsection", WOO_GROUP_PRICING_DOMAIN);
					break;
				case 'advanced' :
					if ( isset( $_POST['wgp-decimals'] ) ) {
						update_option( 'wgp-decimals', $_POST['wgp-decimals'] );
					}
					$alert = __("Saved Advanced subsection", WOO_GROUP_PRICING_DOMAIN);
					break;
				case 'method' :
				default:
					if ( isset( $_POST['wgp-method'] ) ) {
						update_option( 'wgp-method', $_POST['wgp-method'] );
					}
					if ( isset( $_POST['wgp-baseprice'] ) ) {
						update_option( 'wgp-baseprice', $_POST['wgp-baseprice'] );
					}
					if ( isset( $_POST['wgp-ifseveral'] ) ) {
						update_option( 'wgp-ifseveral', $_POST['wgp-ifseveral'] );
					}
					if ( isset( $_POST["wgp-bothpricestext"] ) ) {
						update_option( "wgp-bothpricestext", $_POST[ "wgp-bothpricestext" ] );
					}
					if ( isset( $_POST["wgp-displayoriginal"] ) ) {
						update_option( "wgp-displayoriginal", $_POST["wgp-displayoriginal"] );
					} else {
						update_option( "wgp-displayoriginal",0 );
					}
					if ( isset( $_POST[ "wgp-haveset" ] ) ) {
						update_option( "wgp-haveset", $_POST[ "wgp-haveset" ] );
					}
					$alert = __("Saved Method subsection", WOO_GROUP_PRICING_DOMAIN);
					break;
			}

			// clear product cache
			WooGroupPricing::clear_products_cache();
		}

		if ($alert != "") {
			echo '<div style="background-color: #ffffe0;border: 1px solid #993;padding: 1em;margin-right: 1em;">' . $alert . '</div>';
		}

		$section_title = $section_links_array[$section];

		echo '<h2>' . __( 'Woocommerce Group Pricing', WOO_GROUP_PRICING_DOMAIN ) . '</h2>';
		echo '<hr>';

		$section_links = '';
		foreach( $section_links_array as $sec => $title ) {
			$section_links .= sprintf(
				'<a class="section-link nav-tab %s" href="%s">%s</a>',
				$section == $sec ? 'active nav-tab-active' : '',
				esc_url( add_query_arg( 'section', $sec, admin_url( 'admin.php?page=woogrouppricing' ) ) ),
				$title
			);
		}

		echo '<div class="section-links nav-tab-wrapper">';
		echo $section_links;
		echo '</div>';

		switch( $section ) {
			case 'groups' :
				self::section_groups();
				break;
			case 'advanced' :
				self::section_advanced();
				break;
			case 'method' :
			default :
				self::section_method();
				break;
		}
	}

	/**
	 * Display the Method subsection settings page.
	 */
	protected static function section_method() {
		?>
		<div class="wrap" style="border: 1px solid #ccc; padding:10px;">
		<form method="post" action="">
			<table class="form-table">
				<tr valign="top">
				<th scope="row"><strong><?php echo __( 'Products discount method:', WOO_GROUP_PRICING_DOMAIN ); ?></strong></th>
				<td>
					<select name="wgp-method">
					<?php 
					if ( get_option("wgp-method") == "amount" ) {
					?>
						<option value="rate"><?php echo __( 'Rate', WOO_GROUP_PRICING_DOMAIN ); ?></option>
						<option value="amount" selected="selected"><?php echo __( 'Amount', WOO_GROUP_PRICING_DOMAIN ); ?></option>
					<?php 
					} else {
					?>
						<option value="rate" selected="selected"><?php echo __( 'Rate', WOO_GROUP_PRICING_DOMAIN ); ?></option>
						<option value="amount"><?php echo __( 'Amount', WOO_GROUP_PRICING_DOMAIN ); ?></option>
					<?php 
					}
					?>
					</select>
				</tr>

				<tr valign="top">
				<th scope="row"><strong><?php echo __( 'Apply to:', WOO_GROUP_PRICING_DOMAIN ); ?></strong></th>
				<td>
					<select name="wgp-baseprice">
					<?php 
					if ( get_option("wgp-baseprice") == "sale" ) {
					?>
						<option value="regular"><?php echo __( 'Regular price', WOO_GROUP_PRICING_DOMAIN ); ?></option>
						<option value="sale" selected="selected"><?php echo __( 'Sale price', WOO_GROUP_PRICING_DOMAIN ); ?></option>
					<?php 
					} else {
					?>
						<option value="regular" selected="selected"><?php echo __( 'Regular price', WOO_GROUP_PRICING_DOMAIN ); ?></option>
						<option value="sale"><?php echo __( 'Sale price', WOO_GROUP_PRICING_DOMAIN ); ?></option>
					<?php 
					}
					?>
					</select>
					</br>
					<span class="description"><?php echo __( "If you select <b>Sale Price</b> then if the product has a sale price then discount is applied over this sale price. But if it doesn't, then the discount is applied on the regular price.", 'WOO_GROUP_PRICING_DOMAIN'); ?></span>
				</tr>

				<tr valign="top">
				<th scope="row"><strong><?php echo __( 'Your values are:', WOO_GROUP_PRICING_DOMAIN ); ?></strong></th>
				<td>
					<select name="wgp-haveset">
					<?php 
					switch ( get_option("wgp-haveset") ) {
						case 'amounts':
							?>
							<option value="discounts"><?php echo __( 'Discounts', WOO_GROUP_PRICING_DOMAIN );?></option>
							<option value="amounts" selected="selected"><?php echo __( 'Final values', WOO_GROUP_PRICING_DOMAIN );?></option>
							<?php
							break;
						case 'discounts':
						default:
							?>
							<option value="discounts" selected="selected"><?php echo __( 'Discounts', WOO_GROUP_PRICING_DOMAIN );?></option>
							<option value="amounts"><?php echo __( 'Final values', WOO_GROUP_PRICING_DOMAIN );?></option>
							<?php
							break;
					}
					?>
					</select>
					</br>
					<span class="description"><?php echo __( "If you select <b>Discounts</b> and your value is 0.1, then the group will see 90% of the original price.", 'WOO_GROUP_PRICING_DOMAIN'); ?></span>
					</br>
					<span class="description"><?php echo __( "If you select <b>Final values</b> and your value is 0.1, then the group will see 10% of the original price.", 'WOO_GROUP_PRICING_DOMAIN'); ?></span>
				</tr>

				<tr valign="top">
				<th scope="row"><strong><?php echo __( 'If in several groups:', WOO_GROUP_PRICING_DOMAIN ); ?></strong></th>
				<td>
					<select name="wgp-ifseveral">
					<?php 
					switch ( get_option("wgp-ifseveral") ) {
						case 'last':
						?>
							<option value="higher" ><?php echo __( "Higher discount", WOO_GROUP_PRICING_DOMAIN ); ?></option>
							<option value="lower" ><?php echo __( "Lower discount", WOO_GROUP_PRICING_DOMAIN ); ?></option>
							<option value="first" ><?php echo __( "Higher group ID", WOO_GROUP_PRICING_DOMAIN ); ?></option>
							<option value="last" selected="selected"><?php echo __( "Lower group ID", WOO_GROUP_PRICING_DOMAIN ); ?></option>
						<?php 
							break;
						case 'higher':
						?>
							<option value="higher" selected="selected"><?php echo __( "Higher discount", WOO_GROUP_PRICING_DOMAIN ); ?></option>
							<option value="lower" ><?php echo __( "Lower discount", WOO_GROUP_PRICING_DOMAIN ); ?></option>
							<option value="first" ><?php echo __( "Higher group ID", WOO_GROUP_PRICING_DOMAIN ); ?></option>
							<option value="last" ><?php echo __( "Lower group ID", WOO_GROUP_PRICING_DOMAIN ); ?></option>
						<?php 
							break;
						case 'first':
						?>
							<option value="higher" ><?php echo __( "Higher discount", WOO_GROUP_PRICING_DOMAIN ); ?></option>
							<option value="lower" ><?php echo __( "Lower discount", WOO_GROUP_PRICING_DOMAIN ); ?></option>
							<option value="first" selected="selected" ><?php echo __( "Higher group ID", WOO_GROUP_PRICING_DOMAIN ); ?></option>
							<option value="last" ><?php echo __( "Lower group ID", WOO_GROUP_PRICING_DOMAIN ); ?></option>
						<?php 
							break;
						case 'lower':
						default:
						?>
							<option value="higher" ><?php echo __( "Higher discount", WOO_GROUP_PRICING_DOMAIN ); ?></option>
							<option value="lower" selected="selected"><?php echo __( "Lower discount", WOO_GROUP_PRICING_DOMAIN ); ?></option>
							<option value="first" ><?php echo __( "Higher group ID", WOO_GROUP_PRICING_DOMAIN ); ?></option>
							<option value="last" ><?php echo __( "Lower group ID", WOO_GROUP_PRICING_DOMAIN ); ?></option>
						<?php 
						break;
					}
					?>
					</select>
				</tr>

				<tr valign="top">
				<th scope="row"><strong><?php echo __( 'Display original prices:', WOO_GROUP_PRICING_DOMAIN ); ?></strong></th>
				<td>
					<?php 
					if ( get_option("wgp-displayoriginal", 0) == 1 ) {
						$checked = "checked";
					} else {
						$checked = "";
					}
					?>
					<input type="checkbox" name="wgp-displayoriginal" value="1" <?php echo $checked; ?> >
				</tr>

				<tr valign="top">
				<th scope="row"><strong><?php echo __( 'Both prices text:', WOO_GROUP_PRICING_DOMAIN ); ?></strong></th>
				<td>
					<?php
					$text = get_option( "wgp-bothpricestext", WooGroupPricing::DEFAULT_BOTH_PRICES_TEXT );
					if ( $text == "" ) {
						$text = WooGroupPricing::DEFAULT_BOTH_PRICES_TEXT;
					}
					?>
					<textarea name="wgp-bothpricestext" rows="5" cols="50" ><?php echo $text; ?></textarea>

					<p class="description"><?php echo __( 'You can use the tokens: [original_price], [discounted_price].', WOO_GROUP_PRICING_DOMAIN ); ?></p>
				</tr>

			</table>

			<?php submit_button( __( "Save", WOO_GROUP_PRICING_DOMAIN ) ); ?>
			<?php settings_fields( 'woogrouppricing' ); ?>
		</form>
		</div>
		<?php
	}

	/**
	 * Display the Groups subsection settings page.
	 */
	protected static function section_groups() {
		$groups = WooGroupPricing::get_all_groups();
		?>
		<div class="wrap" style="border: 1px solid #ccc; padding:10px;">
		<form method="post" action="">
			<h3><?php echo __( 'Groups:', WOO_GROUP_PRICING_DOMAIN ); ?></h3>
			<div class="description">Leave empty if no group discount should be applied (default setting).<br>
			Example with rate method: Indicate 0.1 for 10% discounts on every product.
			</div>

			<table class="form-table">
			<?php 
				if ( $groups ) {
					foreach ( $groups as $group ) {
					?>
						<tr valign="top">
						<th scope="row"><?php echo $group->name . ':'; ?></th>
						<td>
							<input type="text" name="wgp-<?php echo $group->group_id;?>" value="<?php echo get_option( "wgp-" . $group->group_id ); ?>" />
						</td>
						</tr>
						<?php
					}
				}
				?>
			</table>

			<?php submit_button( __( "Save", WOO_GROUP_PRICING_DOMAIN ) ); ?>
			<?php settings_fields( 'woogrouppricing' ); ?>
		</form>
		</div>
		<?php
	}

	/**
	 * Display the Advanced subsection settings page.
	 */
	protected static function section_advanced() {
		?>
		<div class="wrap" style="border: 1px solid #ccc; padding:10px;">
		<form method="post" action="">
			<table class="form-table">
				<tr valign="top">
				<th scope="row"><strong><?php echo __( 'Num. decimals:', WOO_GROUP_PRICING_DOMAIN ); ?></strong></th>
				<td>
					<input type="text" name="wgp-decimals" value="<?php echo get_option( 'wgp-decimals', apply_filters( 'woo_group_pricing_num_decimals', 4 ) );?>" />
				</tr>
			</table>

			<?php submit_button( __( "Save", WOO_GROUP_PRICING_DOMAIN ) ); ?>
			<?php settings_fields( 'woogrouppricing' ); ?>
		</form>
		</div>
		<?php 
	}

	/**
	 * Plugin activation work.
	 */
	public static function activate() {
	}

	/**
	 * Plugin deactivation.
	 *
	 */
	public static function deactivate() {
	}

	/**
	 * Plugin uninstall. Delete database table.
	 *
	 */
	public static function uninstall() {
	}

}

WooGroupPricing_Plugin::init();